//
// Created by philipp on 02.01.18.
//

#ifndef NETWORKINGLIB_UTILS_H
#define NETWORKINGLIB_UTILS_H

#include <functional>

namespace networking
{
namespace utils
{

// Generic object which calls a callback-function on destruction.
class RAIIObject
{
public:
    using OnDestructionCallback = std::function<void()>;

    RAIIObject(OnDestructionCallback onDestructionCallback)
        : onDestructionCallback(onDestructionCallback)
    {}

    ~RAIIObject()
    { onDestructionCallback(); }

    RAIIObject(const RAIIObject & other) = delete;

    RAIIObject & operator=(const RAIIObject & other) = delete;

    RAIIObject(RAIIObject && other)
        : onDestructionCallback(other.onDestructionCallback)
    {
        other.onDestructionCallback = []{};
    }

    RAIIObject & operator=(RAIIObject && other)
    {
        onDestructionCallback = other.onDestructionCallback;
        other.onDestructionCallback = []{};
    }

private:
    OnDestructionCallback onDestructionCallback;
};

}
}

#endif //NETWORKINGLIB_UTILS_H
